#include <PalmOS.h>
#include "KioskDA.h"

/* Copyright (c) 2005 Raymond Ingles.

   This is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
   Foundation; either version 2, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.  */

typedef struct {
  UInt16   locked_p; /* are we locked or not? */
  UInt16   hidepw_p; /* hide or display p/w? */
  UInt16   blocksecr_p; /* Don't let Security app run? */
  UInt16   blockpanl_p; /* Don't let Preference panels run? */
  UInt16   beeplock_p; /* Beep when locking/unlocking? */
  UInt16   beeplaunch_p; /* Beep when blocking a launch? */
  UInt16   AppCardNo; /* the card where the app is located */
  LocalID  AppDbID; /* the db id of the app */
  char     passwd_hash[HASH_BIN_SIZE];
} Kiosk_Prefs;

static void EventLoop(Kiosk_Prefs *);
static Boolean HandleEvent(EventPtr, Kiosk_Prefs *);
static void Play_Tone(UInt16 lock_p);
static Boolean ValidatePW(char *input, char *hash);

asm ( " bra start" );

void start()
{
  UInt16 prefs_size;
  Kiosk_Prefs prefs;
  FormPtr form;
  Err err;

  /* Get prefs; are we locked? */
  prefs_size = sizeof(prefs);

  err = PrefGetAppPreferences(HACK_ID, 0, &prefs, &prefs_size, true);

  if (noPreferenceFound == err || (0 == prefs.AppDbID)) {
    /* Some kind of error with the hack. */
    FrmAlert(HACKERR_AL_ID);
    return;
  } else {
    if (prefs.locked_p) {
      /* We're locked. Ask the user for a password to unlock. */
      form = FrmInitForm(KIOSK_GETPASS_F);
      FrmSetActiveForm(form);
      FrmSetControlValue(form, FrmGetObjectIndex(form, HIDEPW_CB_ID),
                         prefs.hidepw_p);
      /* Text is still there, just displayed as squares. */
      FldSetFont(FrmGetObjectPtr(form,
                                 FrmGetObjectIndex(form, PASS_FL_ID)),
                 prefs.hidepw_p ? symbol11Font : stdFont);
      FrmSetFocus(form, FrmGetObjectIndex(form, PASS_FL_ID));
      FrmDrawForm(form);

      EventLoop(&prefs);

      if (!prefs.locked_p) {
        /* We successfully unlocked! */
        FrmEraseForm(form);
        FrmDeleteForm(form);

        if (prefs.beeplock_p) {
          Play_Tone(0);
        }

        /* Display the 'unlock' icon */
        form = FrmInitForm(KIOSK_UNLOCKED_F);
        FrmSetActiveForm(form);
        FrmDrawForm(form);
      }
    } else {
      /* Lock it. */
      prefs.locked_p = 1;

      if (prefs.beeplock_p) {
        Play_Tone(1);
      }

      /* Display the 'lock' icon */
      form = FrmInitForm(KIOSK_LOCKED_F);
      FrmSetActiveForm(form);
      FrmDrawForm(form);
    }
  }
  /* Save prefs */
  PrefSetAppPreferences(HACK_ID, 0, 1, &prefs, sizeof(prefs), true);

  /* Wait half a second. */
  SysTaskDelay(SysTicksPerSecond()/2);

  /* Clear out displayed form. */
  FrmEraseForm(form);
  FrmDeleteForm(form);
}

static Boolean ValidatePW(char *input, char *hash)
{
  char temp_hash[HASH_BIN_SIZE];

  /* Compute the MD5 hash of the password. */
  EncDigestMD5(input, StrLen(input), temp_hash);
  /* Compare it to the saved hash; do they match? */
  return !StrNCompare(temp_hash, hash, HASH_BIN_SIZE);
}

static void EventLoop(Kiosk_Prefs *prefs)
{
    EventType event;
    Boolean done = false;

    do {
        EvtGetEvent(&event, evtWaitForever);
        if (!SysHandleEvent(&event))
            if (!(done = HandleEvent(&event, prefs)))
                FrmHandleEvent(FrmGetActiveForm(), &event);
    } while (done == false);
}

static Boolean HandleEvent(EventPtr event, Kiosk_Prefs *prefs)
{
  Boolean done=false;
  FormPtr form;

  switch (event->eType) {

  case appStopEvent:
    EvtAddEventToQueue(event);
    done = true;
    break;
            /*
        case keyDownEvent:
            ch= event->data.keyDown.chr;
            if (TxtCharIsHardKey(event->data.keyDown.modifiers, event->data.keyDown.chr)) {
                EvtAddEventToQueue(event);
                done= true;
                handled= true;
            }
            break;
            */
  case penDownEvent:
    form= FrmGetActiveForm();
    if (!RctPtInRectangle(event->screenX, event->screenY,
                          &form->window.windowBounds)) {
      /* Tapped outside of our form. Go away. */
      done = true;
    }
    break;

  case ctlSelectEvent:
    switch (event->data.ctlSelect.controlID) {
    case HIDEPW_CB_ID:
      form= FrmGetActiveForm();
      /* Text is still there, just displayed as squares. */
      prefs->hidepw_p = FrmGetControlValue(form,
                                           FrmGetObjectIndex(form,
                                                             HIDEPW_CB_ID));
      FldSetFont(FrmGetObjectPtr(form, FrmGetObjectIndex(form, PASS_FL_ID)),
                 prefs->hidepw_p ? symbol11Font : stdFont);
      break;

    case CANCEL_B_ID:
      done = true;
      break;

    case OK_B_ID:
      form= FrmGetActiveForm();
      /* Validate the password. */
      if (ValidatePW(FldGetTextPtr(FrmGetObjectPtr(form,
                       FrmGetObjectIndex(form, PASS_FL_ID))),
                       prefs->passwd_hash)) {
        prefs->locked_p = 0;
        done = true;
      } else {
        /* display 'bad password' alert. */
        FrmAlert(PWERR_AL_ID);
      }
      break;
    default:
      break;
    }
    break;

  default:
    break;
  }

  return done;
}

static void Play_Tone(UInt16 lock_p)
{
  SndCommandType snd_cmd;

  snd_cmd.cmd = sndCmdFreqDurationAmp;
  snd_cmd.param3 = PrefGetPreference(prefAlarmSoundVolume);
  snd_cmd.param2 = TONE_DURATION;

  if (lock_p) {
    snd_cmd.param1 = TONE_HIGH;
  } else {
    snd_cmd.param1 = TONE_LOW;
  }
  SndDoCmd(0, &snd_cmd, true);
}
